import jp.kobe_u.copris._
import jp.kobe_u.copris.dsl._

object Sugar {
  val name = this.toString
  var help = false
  var quiet = 0
  var verbose = 0
  var dumpFile: String = _
  var options: Set[String] = Set.empty

  def findFirstSolution: Boolean = find
  def findNextSolution: Boolean = findNext
  def findOptSolution: Boolean = findOpt

  def showSolution {
    for (v <- csp.variables)
      print(v + "=" + solution(v) + " ")
    for (v <- csp.bools)
      print(v + "=" + solution(v) + " ")
    println
  }

  def solve {
    var count = 0
    def _showSolution {
      count += 1
      println("BEGIN_solution = " + count)
      showSolution
      println("END_solution = " + count)
    }

    if (dumpFile != null)
      dump(dumpFile)
    if (options.contains("opt")) {
      if (findOptSolution) {
	_showSolution
	println("NumOfOptSolutions >= 1")
      } else {
	println("NumOfOptSolutions = 0")
      }
    } else {
      if (findFirstSolution) {
	_showSolution
	if (options.contains("all")) {
	  while (findNextSolution) {
	    _showSolution
	  }
	  println("NumOfSolutions = " + count)
	} else if (options.contains("multi")) {
	  if (findNextSolution) {
	    if (verbose >= 2)
	      _showSolution
	    println("NumOfSolutions >= 2")
	  } else {
	    println("NumOfSolutions = 1")
	  }
	} else {
	  println("NumOfSolutions >= 1")
	}
      } else {
	println("NumOfSolutions = 0")
      }
    }
  }

  def main(args: Array[String]) = {
    var fileName = ""
    var solverName = ""
    def parseOptions(args: List[String]): List[String] = args match {
      case "-o" :: opt :: rest =>
	{ options = opt.split(",").toSet; parseOptions(rest) }
      case "-s1" :: solver :: rest => {
	solverName = solver
	val satSolver = new sugar.SatSolver1(solver)
	use(new sugar.Solver(csp, satSolver))
	parseOptions(rest)
      }
      case "-s2" :: solver :: rest => {
	solverName = solver
	val satSolver = new sugar.SatSolver2(solver)
	use(new sugar.Solver(csp, satSolver))
	parseOptions(rest)
      }
      case "-smt" :: solver :: rest => {
	solverName = solver
	val smtSolver = new smt.SmtSolver(solver)
	use(new smt.Solver(csp, smtSolver))
	parseOptions(rest)
      }
      case "-q" :: rest =>
	{ quiet += 1; parseOptions(rest) }
      case "-v" :: rest =>
	{ verbose += 1; parseOptions(rest) }
      case "-dump" :: file :: rest =>
	{ dumpFile = file; parseOptions(rest) }
      case "-h" :: rest =>
	{ help = true; parseOptions(rest) }
      case _ =>
	args
    }
    parseOptions(args.toList) match {
      /*
      case Nil if ! help => {
	fileName = "-"
        // TODO (new loader.SugarLoader(csp)).load(Source.stdin)
      }
      */
      case file :: Nil if ! help => {
	fileName = file
        (new loader.SugarLoader(csp)).load(fileName)
      }
      case _ => {
        println("Usage: scala " + name + " [options] [inputFile]")
	println("\t-o options  : Set options ('multi', 'opt', etc.)")
	println("\t-v          : Increase verbosity level")
	println("\t-q          : Increase quiet level")
	println("\t-s1 solver  : Use SAT solver with one argument (clasp, etc.)")
	println("\t-s2 solver  : Use SAT solver with two arguments (minisat, etc.)")
	println("\t-smt solver : Use SMT solver (z3, etc.)")
	println("\t-dump file  : Output Sugar CSP to the specified file")
	println("\t-h          : Show this help message")
	System.exit(1)
      }
    }
    if (verbose >= 1) {
      println("File = " + fileName)
      println("Solver = " + solverName)
      println("Options = " + options.mkString(","))
    }
    solve
    if (verbose >= 2) {
      println("Info  = " + info)
      println("Stats = " + stats)
    }
  }
}
